@@ -167,7 +167,7 @@ int is_authorized(osm_console_t * p_oct)
 	/* allowed to use the console? */
 	p_oct->authorized = !is_remote(p_oct->client_type) ||
 	    hosts_ctl((char *)OSM_DAEMON_NAME, p_oct->client_hn, p_oct->client_ip,
-		      (char *)"STRING_UNKNOWN");
+		      (char *)STRING_UNKNOWN);
 	return p_oct->authorized;
 }
 #endif
@@ -208,8 +208,15 @@ int osm_console_init(osm_subn_opt_t * opt, osm_console_t * p_oct, osm_log_t * p_
 				strerror(errno));
 			return -1;
 		}
-		setsockopt(p_oct->socket, SOL_SOCKET, SO_REUSEADDR,
-			   &optval, sizeof(optval));
+
+		if (setsockopt(p_oct->socket, SOL_SOCKET, SO_REUSEADDR,
+			       &optval, sizeof(optval))) {
+			OSM_LOG(p_log, OSM_LOG_ERROR,
+		                "ERR 4B06: Failed to set socket option: %s\n",
+		                strerror(errno));
+		        return -1;
+		}
+
 		sin.sin_family = AF_INET;
 		sin.sin_port = htons(opt->console_port);
 #ifdef ENABLE_OSM_CONSOLE_SOCKET
