@@ -43,6 +43,8 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_MESH_C
 #include <opensm/osm_switch.h>
 #include <opensm/osm_opensm.h>
 #include <opensm/osm_log.h>
@@ -872,7 +874,7 @@ static void seed_axes(lash_t *p_lash, int sw)
 		}
 	}
 
-	if (osm_log_is_active(p_log, OSM_LOG_DEBUG)) {
+	if (OSM_LOG_IS_ACTIVE_V2(p_log, OSM_LOG_DEBUG)) {
 		char buf[256], *p;
 
 		for (i = 0; i < n; i++) {
@@ -1385,7 +1387,8 @@ static int compare_switches(const void *p1, const void *p2)
 
 	for (i = 0; i < ctx->mesh->dimension; i++) {
 		j = ctx->mesh->dim_order[i];
-		if ((ret = s1->node->coord[j] - s2->node->coord[j]));
+		ret = s1->node->coord[j] - s2->node->coord[j];
+		if (ret)
 			return ret;
 	}
 
@@ -1633,7 +1636,6 @@ int osm_do_mesh_analysis(lash_t *p_lash)
 {
 	osm_log_t *p_log = &p_lash->p_osm->log;
 	mesh_t *mesh;
-	int max_class = -1;
 	int max_class_num = 0;
 	int max_class_type = -1;
 	int i;
@@ -1659,14 +1661,13 @@ int osm_do_mesh_analysis(lash_t *p_lash)
 	 * find dominant switch class
 	 */
 	OSM_LOG(p_log, OSM_LOG_INFO, "found %d node class%s\n",
-		mesh->num_class, (mesh->num_class == 1)? "" : "es");
+		mesh->num_class, (mesh->num_class == 1) ? "" : "es");
 	for (i = 0; i < mesh->num_class; i++) {
 		OSM_LOG(p_log, OSM_LOG_INFO,
 			"class[%d] has %d members with type = %d\n",
 			i, mesh->class_count[i],
 			p_lash->switches[mesh->class_type[i]]->node->type);
 		if (mesh->class_count[i] > max_class_num) {
-			max_class = i;
 			max_class_num = mesh->class_count[i];
 			max_class_type = mesh->class_type[i];
 		}
@@ -1720,7 +1721,7 @@ int osm_do_mesh_analysis(lash_t *p_lash)
 		OSM_LOG(p_log, OSM_LOG_INFO, "%s", buf);
 	}
 
-	if (osm_log_is_active(p_log, OSM_LOG_DEBUG))
+	if (OSM_LOG_IS_ACTIVE_V2(p_log, OSM_LOG_DEBUG))
 		dump_mesh(p_lash);
 
 done:
