@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
- * Copyright (c) 2002-2012 Mellanox Technologies LTD. All rights reserved.
+ * Copyright (c) 2002-2014 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
  * Copyright (c) 2008 Xsigo Systems Inc.  All rights reserved.
  *
@@ -132,6 +132,14 @@ void osm_sa_shutdown(IN osm_sa_t * p_sa)
 	cl_disp_unregister(p_sa->lft_disp_h);
 	cl_disp_unregister(p_sa->sir_disp_h);
 	cl_disp_unregister(p_sa->mft_disp_h);
+
+	if (p_sa->p_set_disp) {
+		cl_disp_unregister(p_sa->mcmr_set_disp_h);
+		cl_disp_unregister(p_sa->infr_set_disp_h);
+		cl_disp_unregister(p_sa->sr_set_disp_h);
+		cl_disp_unregister(p_sa->gir_set_disp_h);
+	}
+
 	osm_sa_mad_ctrl_destroy(&p_sa->mad_ctrl);
 
 	OSM_LOG_EXIT(p_sa->p_log);
@@ -152,7 +160,9 @@ ib_api_status_t osm_sa_init(IN osm_sm_t * p_sm, IN osm_sa_t * p_sa,
 			    IN osm_subn_t * p_subn, IN osm_vendor_t * p_vendor,
 			    IN osm_mad_pool_t * p_mad_pool,
 			    IN osm_log_t * p_log, IN osm_stats_t * p_stats,
-			    IN cl_dispatcher_t * p_disp, IN cl_plock_t * p_lock)
+			    IN cl_dispatcher_t * p_disp,
+			    IN cl_dispatcher_t * p_set_disp,
+			    IN cl_plock_t * p_lock)
 {
 	ib_api_status_t status;
 
@@ -164,13 +174,14 @@ ib_api_status_t osm_sa_init(IN osm_sm_t * p_sm, IN osm_sa_t * p_sa,
 	p_sa->p_mad_pool = p_mad_pool;
 	p_sa->p_log = p_log;
 	p_sa->p_disp = p_disp;
+	p_sa->p_set_disp = p_set_disp;
 	p_sa->p_lock = p_lock;
 
 	p_sa->state = OSM_SA_STATE_READY;
 
 	status = osm_sa_mad_ctrl_init(&p_sa->mad_ctrl, p_sa, p_sa->p_mad_pool,
 				      p_sa->p_vendor, p_subn, p_log, p_stats,
-				      p_disp);
+				      p_disp, p_set_disp);
 	if (status != IB_SUCCESS)
 		goto Exit;
 
@@ -277,6 +288,36 @@ ib_api_status_t osm_sa_init(IN osm_sm_t * p_sm, IN osm_sa_t * p_sa,
 	if (p_sa->mft_disp_h == CL_DISP_INVALID_HANDLE)
 		goto Exit;
 
+	/*
+	 * When p_set_disp is defined, it means that we use different dispatcher
+	 * for SA Set requests, and we need to register handlers for it.
+	 */
+	if (p_set_disp) {
+		p_sa->gir_set_disp_h =
+		    cl_disp_register(p_set_disp, OSM_MSG_MAD_GUIDINFO_RECORD,
+				     osm_gir_rcv_process, p_sa);
+		if (p_sa->gir_set_disp_h == CL_DISP_INVALID_HANDLE)
+			goto Exit;
+
+		p_sa->mcmr_set_disp_h =
+		    cl_disp_register(p_set_disp, OSM_MSG_MAD_MCMEMBER_RECORD,
+				     osm_mcmr_rcv_process, p_sa);
+		if (p_sa->mcmr_set_disp_h == CL_DISP_INVALID_HANDLE)
+			goto Exit;
+
+		p_sa->sr_set_disp_h =
+		    cl_disp_register(p_set_disp, OSM_MSG_MAD_SERVICE_RECORD,
+				     osm_sr_rcv_process, p_sa);
+		if (p_sa->sr_set_disp_h == CL_DISP_INVALID_HANDLE)
+			goto Exit;
+
+		p_sa->infr_set_disp_h =
+		    cl_disp_register(p_set_disp, OSM_MSG_MAD_INFORM_INFO,
+				     osm_infr_rcv_process, p_sa);
+		if (p_sa->infr_set_disp_h == CL_DISP_INVALID_HANDLE)
+			goto Exit;
+	}
+
 	status = IB_SUCCESS;
 Exit:
 	OSM_LOG_EXIT(p_log);
@@ -401,9 +442,11 @@ void osm_sa_respond(osm_sa_t *sa, osm_madw_t *madw, size_t attr_size,
 	 */
 	if (sa_mad->method == IB_MAD_METHOD_GET && num_rec > 1) {
 		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4C05: "
-			"Got %u records for SubnAdmGet(%s) comp_mask 0x%016" PRIx64 "\n",
+			"Got %u records for SubnAdmGet(%s) comp_mask 0x%016" PRIx64
+			"from requester LID %u\n",
 			num_rec, ib_get_sa_attr_str(sa_mad->attr_id),
-			cl_ntoh64(sa_mad->comp_mask));
+			cl_ntoh64(sa_mad->comp_mask),
+			cl_ntoh16(madw->mad_addr.dest_lid));
 		osm_sa_send_error(sa, madw, IB_SA_MAD_STATUS_TOO_MANY_RECORDS);
 		goto Exit;
 	}
@@ -524,7 +567,14 @@ opensm_dump_to_file(osm_opensm_t * p_osm, const char *file_name,
 		return -1;
 	}
 
-	chmod(path_tmp, S_IRUSR | S_IWUSR);
+	if (chmod(path_tmp, S_IRUSR | S_IWUSR)) {
+		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 4C0C: "
+			"cannot change access permissions of file "
+			"\'%s\' : %s\n",
+			path_tmp, strerror(errno));
+		fclose(file);
+		return -1;
+	}
 
 	dump_func(p_osm, file);
 
@@ -750,7 +800,6 @@ static void sa_dump_all_sa(osm_opensm_t * p_osm, FILE * file)
 	dump_context.p_osm = p_osm;
 	dump_context.file = file;
 	OSM_LOG(&p_osm->log, OSM_LOG_DEBUG, "Dump guidinfo\n");
-	cl_plock_acquire(&p_osm->lock);
 	cl_qmap_apply_func(&p_osm->subn.port_guid_tbl,
 			   sa_dump_one_port_guidinfo, &dump_context);
 	OSM_LOG(&p_osm->log, OSM_LOG_DEBUG, "Dump multicast\n");
@@ -764,17 +813,21 @@ static void sa_dump_all_sa(osm_opensm_t * p_osm, FILE * file)
 	OSM_LOG(&p_osm->log, OSM_LOG_DEBUG, "Dump services\n");
 	cl_qlist_apply_func(&p_osm->subn.sa_sr_list,
 			    sa_dump_one_service, &dump_context);
-	cl_plock_release(&p_osm->lock);
 }
 
 int osm_sa_db_file_dump(osm_opensm_t * p_osm)
 {
 	int res = 1;
+
+	cl_plock_acquire(&p_osm->lock);
 	if (p_osm->sa.dirty) {
 		res = opensm_dump_to_file(
 			p_osm, "opensm-sa.dump", sa_dump_all_sa);
-		p_osm->sa.dirty = FALSE;
+		if (!res)
+			p_osm->sa.dirty = FALSE;
 	}
+	cl_plock_release(&p_osm->lock);
+
 	return res;
 }
 
@@ -782,8 +835,7 @@ int osm_sa_db_file_dump(osm_opensm_t * p_osm)
  *  SA DB Loader
  */
 static osm_mgrp_t *load_mcgroup(osm_opensm_t * p_osm, ib_net16_t mlid,
-				ib_member_rec_t * p_mcm_rec,
-				unsigned well_known)
+				ib_member_rec_t * p_mcm_rec)
 {
 	ib_net64_t comp_mask;
 	osm_mgrp_t *p_mgrp;
@@ -816,8 +868,7 @@ static osm_mgrp_t *load_mcgroup(osm_opensm_t * p_osm, ib_net16_t mlid,
 			cl_ntoh64(p_mcm_rec->mgid.unicast.prefix),
 			cl_ntoh64(p_mcm_rec->mgid.unicast.interface_id));
 		p_mgrp = NULL;
-	} else if (well_known)
-		p_mgrp->well_known = TRUE;
+	}
 
 _out:
 	cl_plock_release(&p_osm->lock);
@@ -1039,13 +1090,29 @@ static int unpack_string64(char *p, uint8_t * buf)
 	p += _ret; \
 }
 
+static void sa_db_file_load_handle_mgrp(osm_opensm_t * p_osm,
+					osm_mgrp_t * p_mgrp)
+{
+	/* decide whether to delete the mgrp object or not */
+	if (p_mgrp->full_members == 0 && !p_mgrp->well_known) {
+		OSM_LOG(&p_osm->log, OSM_LOG_VERBOSE,
+			"Closing MC group 0x%016" PRIx64 ":0x%016" PRIx64
+			" - no full members were added to not well known "
+			"group\n",
+			cl_ntoh64(p_mgrp->mcmember_rec.mgid.unicast.prefix),
+			cl_ntoh64(p_mgrp->mcmember_rec.mgid.unicast.interface_id));
+		osm_mgrp_cleanup(&p_osm->subn, p_mgrp);
+	}
+}
+
 int osm_sa_db_file_load(osm_opensm_t * p_osm)
 {
 	char line[1024];
 	char *file_name;
 	FILE *file;
 	int ret = 0;
-	osm_mgrp_t *p_mgrp = NULL;
+	osm_mgrp_t *p_next_mgrp = NULL;
+	osm_mgrp_t *p_prev_mgrp = NULL;
 	unsigned rereg_clients = 0;
 	unsigned lineno;
 
@@ -1092,14 +1159,11 @@ int osm_sa_db_file_load(osm_opensm_t * p_osm)
 		if (!strncmp(p, "MC Group", 8)) {
 			ib_member_rec_t mcm_rec;
 			ib_net16_t mlid;
-			unsigned well_known = 0;
 
-			p_mgrp = NULL;
+			p_next_mgrp = NULL;
 			memset(&mcm_rec, 0, sizeof(mcm_rec));
 
 			PARSE_AHEAD(p, net16, " 0x", &mlid);
-			if (strstr(p, "well known"))
-				well_known = 1;
 			PARSE_AHEAD(p, net64, " mgid=0x",
 				    &mcm_rec.mgid.unicast.prefix);
 			PARSE_AHEAD(p, net64, ":0x",
@@ -1122,13 +1186,12 @@ int osm_sa_db_file_load(osm_opensm_t * p_osm)
 			PARSE_AHEAD(p, net8, " proxy_join=0x", &val);
 			mcm_rec.proxy_join = val;
 
-			p_mgrp = load_mcgroup(p_osm, mlid, &mcm_rec,
-					      well_known);
-			if (!p_mgrp)
+			p_next_mgrp = load_mcgroup(p_osm, mlid, &mcm_rec);
+			if (!p_next_mgrp)
 				rereg_clients = 1;
 			if (cl_ntoh16(mlid) > p_osm->sm.mlids_init_max)
 				p_osm->sm.mlids_init_max = cl_ntoh16(mlid);
-		} else if (p_mgrp && !strncmp(p, "mcm_port", 8)) {
+		} else if (p_next_mgrp && !strncmp(p, "mcm_port", 8)) {
 			ib_member_rec_t mcmr;
 			ib_net64_t guid;
 			osm_port_t *port;
@@ -1145,16 +1208,16 @@ int osm_sa_db_file_load(osm_opensm_t * p_osm)
 			guid = mcmr.port_gid.unicast.interface_id;
 			port = osm_get_port_by_alias_guid(&p_osm->subn, guid);
 			if (port &&
-			    cl_qmap_get(&p_mgrp->mcm_port_tbl, guid) ==
-			    cl_qmap_end(&p_mgrp->mcm_port_tbl) &&
+			    cl_qmap_get(&p_next_mgrp->mcm_port_tbl, guid) ==
+			    cl_qmap_end(&p_next_mgrp->mcm_port_tbl) &&
 			    !osm_mgrp_add_port(&p_osm->subn, &p_osm->log,
-						p_mgrp, port, &mcmr, proxy))
+						p_next_mgrp, port, &mcmr, proxy))
 				rereg_clients = 1;
 		} else if (!strncmp(p, "Service Record:", 15)) {
 			ib_service_record_t s_rec;
 			uint32_t modified_time, lease_period;
 
-			p_mgrp = NULL;
+			p_next_mgrp = NULL;
 			memset(&s_rec, 0, sizeof(s_rec));
 
 			PARSE_AHEAD(p, net64, " id=0x", &s_rec.service_id);
@@ -1198,7 +1261,7 @@ int osm_sa_db_file_load(osm_opensm_t * p_osm)
 			osm_mad_addr_t rep_addr;
 			ib_net16_t val16;
 
-			p_mgrp = NULL;
+			p_next_mgrp = NULL;
 			memset(&i_rec, 0, sizeof(i_rec));
 			memset(&rep_addr, 0, sizeof(rep_addr));
 
@@ -1253,7 +1316,7 @@ int osm_sa_db_file_load(osm_opensm_t * p_osm)
 			ib_guidinfo_record_t gi_rec;
 			ib_net64_t base_guid;
 
-			p_mgrp = NULL;
+			p_next_mgrp = NULL;
 			memset(&gi_rec, 0, sizeof(gi_rec));
 
 			PARSE_AHEAD(p, net64, " base_guid=0x", &base_guid);
@@ -1280,8 +1343,25 @@ int osm_sa_db_file_load(osm_opensm_t * p_osm)
 			if (load_guidinfo(p_osm, base_guid, &gi_rec))
 				rereg_clients = 1;
 		}
+
+		/*
+		 * p_next_mgrp points to the multicast group now being parsed.
+		 * p_prev_mgrp points to the last multicast group we parsed.
+		 * We decide whether to keep or delete each multicast group
+		 * only when we finish parsing it's member records. if the
+		 * group has full members, or it is a "well known group" we
+		 * keep it.
+		 */
+		if (p_prev_mgrp != p_next_mgrp) {
+			if (p_prev_mgrp)
+				sa_db_file_load_handle_mgrp(p_osm, p_prev_mgrp);
+			p_prev_mgrp = p_next_mgrp;
+		}
 	}
 
+	if (p_next_mgrp)
+		sa_db_file_load_handle_mgrp(p_osm, p_prev_mgrp);
+
 	/*
 	 * If loading succeeded, do whatever 'no_clients_rereg' says.
 	 * If loading failed at some point, turn off the 'no_clients_rereg'
