@@ -1,8 +1,8 @@
 /*
  * Copyright (c) 2004-2008 Voltaire, Inc. All rights reserved.
- * Copyright (c) 2002-2009 Mellanox Technologies LTD. All rights reserved.
+ * Copyright (c) 2002-2015 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
- * Copyright (c) 2009-2011 ZIH, TU Dresden, Federal Republic of Germany. All rights reserved.
+ * Copyright (c) 2009-2015 ZIH, TU Dresden, Federal Republic of Germany. All rights reserved.
  * Copyright (C) 2012-2013 Tokyo Institute of Technology. All rights reserved.
  *
  * This software is available to you under a choice of one of two
@@ -95,6 +95,7 @@ typedef struct vertex {
 	uint32_t heap_id;
 	/* for LFT writing and debug */
 	osm_switch_t *sw;	/* selfpointer */
+	boolean_t dropped;	/* indicate dropped switches (w/ ucast cache) */
 } vertex_t;
 
 typedef struct binary_heap {
@@ -161,6 +162,7 @@ static inline void set_default_vertex(vertex_t * vertex)
 	vertex->state = UNDISCOVERED;
 	vertex->heap_id = 0;
 	vertex->sw = NULL;
+	vertex->dropped = FALSE;
 }
 
 static inline void set_default_cdg_node(cdg_node_t * node)
@@ -338,7 +340,7 @@ static void heap_free(binary_heap_t * heap)
 /**********************************************************************
  **********************************************************************/
 
-/************ helper functions to save src/dest X vl kombination ******
+/************ helper functions to save src/dest X vl combination ******
  **********************************************************************/
 /* compare function of two lids for stdlib qsort */
 static int cmp_lids(const void *l1, const void *l2)
@@ -375,7 +377,7 @@ static inline int64_t vltable_get_lidindex(ib_net16_t * key, vltable_t * vltable
 		return -1;
 }
 
-/* get virtual lane from src lid X dest lid kombination;
+/* get virtual lane from src lid X dest lid combination;
    return -1 for invalid lids
 */
 static int32_t vltable_get_vl(vltable_t * vltable, ib_net16_t slid, ib_net16_t dlid)
@@ -1258,7 +1260,7 @@ static int dfsssp_build_graph(void *context)
 		head = link;
 		head->next = NULL;
 
-		/* add SP0 to number of CA conneted to a switch */
+		/* add SP0 to number of CA connected to a switch */
 		lmc = osm_node_get_lmc(sw->p_node, 0);
 		adj_list[i].num_hca += (1 << lmc);
 
@@ -1558,6 +1560,13 @@ static int update_lft(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 					  (p_sw->p_node)));
 
 			p = osm_node_get_physp_ptr(p_sw->p_node, port);
+			if (!p) {
+				OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
+					"ERR AD0A: Physical port %d of Node GUID 0x%"
+					PRIx64 "not found\n", port,
+					cl_ntoh64(osm_node_get_node_guid(p_sw->p_node)));
+				return 1;
+			}
 
 			/* we would like to optionally ignore this port in equalization
 			   as in the case of the Mellanox Anafa Internal PCI TCA port
@@ -1574,7 +1583,7 @@ static int update_lft(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 		}
 
 		/* to support lmc > 0 the functions alloc_ports_priv, free_ports_priv, find_and_add_remote_sys
-		   from minhop aren't needed cause osm_switch_recommend_path is implicit calulated
+		   from minhop aren't needed cause osm_switch_recommend_path is implicitly calculated
 		   for each LID pair thru dijkstra;
 		   for each port the dijkstra algorithm calculates (max_lid_ho - min_lid_ho)-times maybe
 		   disjoint routes to spread the bandwidth -> diffent routes for one port and lmc>0
@@ -1586,7 +1595,7 @@ static int update_lft(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 			/* update the number of path routing thru this port */
 			osm_switch_count_path(p_sw, port);
 		}
-		/* set te hop count from this switch to the lid */
+		/* set the hop count from this switch to the lid */
 		ret = osm_switch_set_hops(p_sw, lid, port, hops);
 		if (ret != CL_SUCCESS)
 			OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
@@ -1632,6 +1641,9 @@ static void reset_mgrp_membership(vertex_t * adj_list, uint32_t adj_list_size)
 	uint32_t i = 0;
 
 	for (i = 1; i < adj_list_size; i++) {
+		if (adj_list[i].dropped)
+			continue;
+
 		adj_list[i].sw->is_mc_member = 0;
 		adj_list[i].sw->num_of_mcm = 0;
 	}
@@ -1657,6 +1669,9 @@ static int update_mcft(osm_sm_t * p_sm, vertex_t * adj_list,
 	OSM_LOG_ENTER(p_sm->p_log);
 
 	for (i = 1; i < adj_list_size; i++) {
+		if (adj_list[i].dropped)
+			continue;
+
 		p_sw = adj_list[i].sw;
 		OSM_LOG(p_sm->p_log, OSM_LOG_VERBOSE,
 			"Processing switch 0x%016" PRIx64
@@ -1759,7 +1774,7 @@ static void update_weights(osm_ucast_mgr_t * p_mgr, vertex_t * adj_list,
 	OSM_LOG_EXIT(p_mgr->p_log);
 }
 
-/* get the larges number of virtual lanes which is supported by all switches
+/* get the largest number of virtual lanes which is supported by all switches
    in the subnet
 */
 static uint8_t get_avail_vl_in_subn(osm_ucast_mgr_t * p_mgr)
@@ -1956,7 +1971,7 @@ static int dfsssp_remove_deadlocks(dfsssp_context_t * dfsssp_ctx)
 								"ERR AD14: cannot allocate memory for cdg node or link in update_channel_dep_graph(...)\n");
 							goto ERROR;
 						}
-						/* add the <s,d> kombination / coresponding virtual lane to the VL table */
+						/* add the <s,d> combination / corresponding virtual lane to the VL table */
 						vltable_insert
 						    (srcdest2vl_table,
 						     cl_hton16(slid),
@@ -2517,24 +2532,68 @@ static ib_api_status_t dfsssp_do_mcast_routing(void * context,
 	uint32_t adj_list_size = dfsssp_ctx->adj_list_size;
 	cl_qlist_t mcastgrp_port_list;
 	cl_qmap_t mcastgrp_port_map;
-	osm_switch_t *root_sw = NULL;
+	osm_switch_t *root_sw = NULL, *p_sw = NULL;
 	osm_port_t *port = NULL;
 	ib_net16_t lid = 0;
-	uint32_t err = 0, num_ports = 0;
+	uint32_t err = 0, num_ports = 0, i = 0;
+	ib_net64_t guid = 0;
 	ib_api_status_t status = IB_SUCCESS;
 
 	OSM_LOG_ENTER(sm->p_log);
 
+	/* using the ucast cache feature with dfsssp might mean that a leaf sw
+	   got removed (and got back) without calling dfsssp_build_graph
+	   and therefore the adj_list (and pointers to osm's internal switches)
+	   could be outdated (here we have no knowledge if it has happened, so
+	   unfortunately a check is necessary... still better than rebuilding
+	   adj_list every time we arrive here)
+	 */
+	if (p_mgr->p_subn->opt.use_ucast_cache && p_mgr->cache_valid) {
+		for (i = 1; i < adj_list_size; i++) {
+			guid = cl_hton64(adj_list[i].guid);
+			p_sw = osm_get_switch_by_guid(p_mgr->p_subn, guid);
+			if (p_sw) {
+				/* check if switch came back from the dead */
+				if (adj_list[i].dropped)
+					adj_list[i].dropped = FALSE;
+
+				/* verify that sw object has not been moved
+				   (this can happen for a leaf switch, if it
+				   was dropped and came back later without a
+				   rerouting), otherwise we have to update
+				   dfsssp's internal switch list with the new
+				   sw pointer
+				 */
+				if (p_sw == adj_list[i].sw)
+					continue;
+				else
+					adj_list[i].sw = p_sw;
+			} else {
+				/* if a switch from adj_list is not in the
+				   sw_guid_tbl anymore, then the only reason is
+				   that it was a leaf switch and opensm dropped
+				   it without calling a rerouting
+				   -> calling dijkstra is no problem, since it
+				      is a leaf and different from root_sw
+				   -> only update_mcft and reset_mgrp_membership
+				      need to be aware of these dropped switches
+				 */
+				if (!adj_list[i].dropped)
+					adj_list[i].dropped = TRUE;
+			}
+		}
+	}
+
 	/* create a map and a list of all ports which are member in the mcast
 	   group; map for searching elements and list for iteration
 	 */
 	if (osm_mcast_make_port_list_and_map(&mcastgrp_port_list,
 					     &mcastgrp_port_map, mbox)) {
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR AD50: "
-                        "Insufficient memory to make port list\n");
-                status = IB_ERROR;
-                goto Exit;
-        }
+			"Insufficient memory to make port list\n");
+		status = IB_ERROR;
+		goto Exit;
+	}
 
 	num_ports = cl_qlist_count(&mcastgrp_port_list);
 	if (num_ports < 2) {
